/*
 * BYOND String Utilities
 * ObjectReference.h
 *
 *  Created on: 15 Jan 2011
 *      Author: Stephen Badger (Stephen001)
 */

#ifndef OBJECTREFERENCE_H_
#define OBJECTREFERENCE_H_

#include <map>
#include <vector>
#include <iostream>
#include <string>

/**
 * A reference to an object. This class is used for tying together the C++ class
 * and DM datum objects. This is usually used in the DataObject structures, to
 * access a static store of DataObjects on the DLL side.
 */
class Reference {
public:
	/**
	 * Constructs the reference. Currently this creates temporaries, which it
	 * should really do.
	 *
	 * \param name The name of the class being referenced.
	 * \param id The reference ID, used in the DataObject structures.
	 */
	Reference(std::string name, unsigned long id);
	/**
	 * Destructor. I don't really expect this class to be sub-classed.
	 */
	~Reference();
	/**
	 * Handy friend function to read references from streams.
	 */
	friend std::istream & operator>>(std::istream & stream, Reference & ref);
	/**
	 * Handy friend function to write references to streams.
	 */
	friend std::ostream & operator<<(std::ostream & stream, const Reference & ref);
	/**
	 * Gets the class this references.
	 *
	 * \return The class name this references.
	 */
	const std::string & name() const;
	/**
	 * Gets the reference ID.
	 *
	 * \return The reference's ID.
	 */
	unsigned long id() const;
private:
	std::string __name;
	unsigned long __id;
};

/**
 * A data holding object. This should probably be renamed to BYONDObject or something,
 * as that's more fitting. Basically it is the base-class of objects that have a DM
 * datum partner.
 */
class DataObject {
public:
	/**
	 * Base constructor, does nothing.
	 */
	DataObject() {};
	/**
	 * Base destructor, does nothing.
	 */
	virtual ~DataObject() {};
	/**
	 * Adds a new DataObject to the static storage.
	 *
	 * \param name The class name.
	 * \param object The object to map.
	 * \return A reference to the object, to be exported to DM.
	 */
	static Reference add_new_object(std::string & name, DataObject * object);
	/**
	 * Gets an object from it's reference.
	 *
	 * \param ref The reference to resolve.
	 * \return The data object being referenced.
	 */
	static DataObject * get_from_reference(const Reference & ref);
	/**
	 * Destroys an object and it's mapping.
	 *
	 * \param ref The reference to the object.
	 */
	static void destroy_from_reference(const Reference & ref);
private:
	static std::map<std::string, std::vector<DataObject *> * > data;
};

/**
 * Gets a reference from some arbitrary C style character string. This is
 * pretty handy for dealing with DM's character string inputs.
 * \param text The text to read from.
 * \param A new reference.
 */
Reference get_reference(char * text);

#endif /* OBJECTREFERENCE_H_ */
